%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "parser.h"
%}

%term NAME INT_CONST DOUBLE_CONST STRING_CONST
%term WHILE ENDWHILE IF ENDIF ELSE ELSEIF TYPE BREAK CONTINUE
%term SM CM LBR RBR LPAR RPAR ILLEGAL
%right ASG
%nonassoc RELOP
%left LOR
%left LAND
%left LNOT
%left PLUS MINUS
%left MUL DIV MOD
%nonassoc UMINUS

%%

program : stms
        ;

stms    : stm                   { printf("stms: stm\n"); }
        | stms stm              { printf("stms: stms stm\n"); }
        ;

stm     : decl                  { printf("stm: decl\n"); }
        | oper                  { printf("stm: oper\n"); }
        | error SM              { printf("stm: error SM\n"); }
        ;

decl    : TYPE defs SM          { printf("decl\n"); }
        ;

defs    : def                   { printf("def\n"); }
        | defs CM def           { printf("defs CM def\n"); }
        ;

def     : NAME                  { printf("name\n"); }
        | NAME LBR expr RBR     { printf("array\n"); }
        ;

oper    : assign
        | if
        | while
        | BREAK SM
        | CONTINUE SM
        ;

assign  : lval ASG expr SM
        ;

lval    : NAME
        | NAME LBR expr RBR
        ;

expr    : expr PLUS expr
        | expr MINUS expr
        | expr MUL expr
        | expr DIV expr
        | expr MOD expr
        | MINUS expr %prec UMINUS
        | NAME
        | NAME LBR expr RBR
        | INT_CONST
        | DOUBLE_CONST
        | STRING_CONST
        | LPAR expr RPAR
        ;

if      : ifh stms ENDIF        { printf("if: ifh stmd ENDIF\n"); }
        | ifh stms ELSE stms ENDIF
        | ifh stms elseifs ENDIF
        | ifh stms elseifs ELSE stms ENDIF
        ;

ifh     : IF LPAR lexpr RPAR    { printf("ifh\n"); }
        ;

elseifs : elseif
        | elseifs elseif
        ;

elseif  : elseifh stms
        ;

elseifh : ELSEIF LPAR lexpr RPAR
        ;

lexpr   : expr RELOP expr       { printf("lexpr: expr RELOP expr\n"); }
        | lexpr LOR lexpr
        | lexpr LAND lexpr
        | LNOT lexpr
        | LPAR expr RPAR
        ;

while   : whileh stms ENDWHILE  { printf("while\n"); }
        ;

whileh  : WHILE LPAR lexpr RPAR { printf("whileh\n"); }
        ;

%%

int main() {
    yyparse();
    return 0;
}

int yyerror(const char *s) {
    printf("%s\n", s);
    return 0;
}
