#ifndef STRPOOL_H
#define STRPOOL_H

#include <string.h>

class L1ListHeader {
public:
    L1ListHeader *next;

    L1ListHeader(): next(0) {}
    L1ListHeader(L1ListHeader *n): next(n) {}

    void link(L1ListHeader *n) { next = n; }
    virtual ~L1ListHeader() {}
};

class L1List {
public:
    L1ListHeader head;

    mutable L1ListHeader* pointerPrev;
    mutable L1ListHeader* pointerNext;

    L1List():
        pointerPrev(&head),
        pointerNext(0)
    {}

    void moveToBeg() const {
        pointerPrev = (L1ListHeader*) &head;
        pointerNext = head.next;
    }

    void moveForward() const {
        if (pointerNext != 0)
            pointerNext = pointerNext->next;
    }

    // Add an element before the pointer.
    // Element must be created in the heap using the "new" operator
    void insertBefore(L1ListHeader* element) {
        element->link(pointerNext);
        pointerPrev->link(element);
    }
    void removeAll();
    ~L1List() { removeAll(); }
};

class StrDef: public L1ListHeader {
public:
    char *str;
    StrDef():
        str(0)
    {}

    StrDef(const StrDef& s):
        str(0)
    {
        if (s.str != 0) {
            int len = strlen(s.str);
            str = new char[len + 1];
            memmove(str, s.str, len+1);
        }
    }

    StrDef(const char *s):
        str(0)
    {
        if (s != 0) {
            int len = strlen(s);
            str = new char[len + 1];
            memmove(str, s, len+1);
        }
    }

    StrDef& operator=(const StrDef& s) {
        delete[] str; str = 0;
        if (s.str != 0) {
            int len = strlen(s.str);
            str = new char[len + 1];
            memmove(str, s.str, len+1);
        }
        return *this;
    }

    operator const char*() const {
        return str;
    }

    virtual ~StrDef() {
        delete [] str;
    }
};

class StrPool: public L1List {
public:
    StrDef* findString(const char *str) const;
    const char *addString(const char *str);
};

#endif
